//
// zedit - simple text editor
// Copyright (C) 2004 Bryan Beicker <tokiko@tokiko.net>
//
// This file is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License (Version 2)
// as published by the Free Software Foundation.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include <gtk/gtk.h>		// gtk
#include "main.h"
#include "zedit.xpm"
#include "new.xpm"
#include "open.xpm"
#include "save.xpm"
#include "save_as.xpm"
#include "about.xpm"


GtkWidget * edit_create()
{
	edit = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(edit), "zedit");
	gtk_widget_set_size_request(edit, 700, 500);
	GTK_WINDOW(edit)->allow_shrink = TRUE;

	GdkPixbuf * pixbuf;
	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) zedit_xpm);
	gtk_window_set_icon(GTK_WINDOW(edit), pixbuf);

	GtkWidget * table;
	table = gtk_table_new(2, 1, FALSE);
	gtk_widget_show(table);
	gtk_container_add(GTK_CONTAINER(edit), table);

	GtkWidget * toolbar;
	toolbar = gtk_toolbar_new();
	gtk_widget_show(toolbar);
	gtk_table_attach(GTK_TABLE(table), toolbar, 0, 1, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);

	GtkTooltips * tips;
	tips = gtk_tooltips_new();
	
	GdkPixbuf * new_pixbuf;
	GtkWidget * new_image;
	new_pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) new_xpm);
	new_image = gtk_image_new_from_pixbuf(new_pixbuf);
	gtk_widget_show(new_image);

	GtkWidget * new_button;
	new_button = (GtkWidget *) gtk_tool_button_new(new_image, "new");
	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(new_button), GTK_TOOLTIPS(tips), "new", NULL);
	gtk_widget_show(new_button);
	gtk_container_add(GTK_CONTAINER(toolbar), new_button);

	GdkPixbuf * open_pixbuf;
	GtkWidget * open_image;
	open_pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) open_xpm);
	open_image = gtk_image_new_from_pixbuf(open_pixbuf);
	gtk_widget_show(open_image);

	GtkWidget * open_button;
	open_button = (GtkWidget *) gtk_tool_button_new(open_image, "open");
	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(open_button), GTK_TOOLTIPS(tips), "open", NULL);
	gtk_widget_show(open_button);
	gtk_container_add(GTK_CONTAINER(toolbar), open_button);

	GdkPixbuf * save_pixbuf;
	GtkWidget * save_image;
	save_pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) save_xpm);
	save_image = gtk_image_new_from_pixbuf(save_pixbuf);
	gtk_widget_show(save_image);

	GtkWidget * save_button;
	save_button = (GtkWidget *) gtk_tool_button_new(save_image, "save");
	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(save_button), GTK_TOOLTIPS(tips), "save", NULL);
	gtk_widget_show(save_button);
	gtk_container_add(GTK_CONTAINER(toolbar), save_button);

	GdkPixbuf * save_as_pixbuf;
	GtkWidget * save_as_image;
	save_as_pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) save_as_xpm);
	save_as_image = gtk_image_new_from_pixbuf(save_as_pixbuf);
	gtk_widget_show(save_as_image);

	GtkWidget * save_as_button;
	save_as_button = (GtkWidget *) gtk_tool_button_new(save_as_image, "save as");
	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(save_as_button), GTK_TOOLTIPS(tips), "save as", NULL);
	gtk_widget_show(save_as_button);
	gtk_container_add(GTK_CONTAINER(toolbar), save_as_button);

	GdkPixbuf * about_pixbuf;
	GtkWidget * about_image;
	about_pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) about_xpm);
	about_image = gtk_image_new_from_pixbuf(about_pixbuf);
	gtk_widget_show(about_image);

	GtkWidget * about_button;
	about_button = (GtkWidget *) gtk_tool_button_new(about_image, "about");
	gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(about_button), GTK_TOOLTIPS(tips), "about", NULL);
	gtk_widget_show(about_button);
	gtk_container_add(GTK_CONTAINER(toolbar), about_button);

	GtkWidget * scroll;
	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scroll);
	gtk_table_attach(GTK_TABLE(table), scroll, 0, 1, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	view = gtk_text_view_new();
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
	gtk_widget_show(view);
	gtk_container_add(GTK_CONTAINER(scroll), view);
	gtk_widget_grab_focus(view);

	PangoFontDescription * font_desc;
	font_desc = pango_font_description_from_string("Monospace 10");
	gtk_widget_modify_font(view, font_desc);
	pango_font_description_free(font_desc);
	
	GtkAccelGroup * accel_group;
	GClosure * closure;
	guint key;
	GdkModifierType mods;
	accel_group = gtk_accel_group_new();
	
	closure = g_cclosure_new(G_CALLBACK(accel_ctrl_n), NULL, NULL);
	gtk_accelerator_parse("<CTRL>N", &key, &mods);
	gtk_accel_group_connect(accel_group, key, mods, GTK_ACCEL_VISIBLE, closure);
	g_closure_unref(closure);

	closure = g_cclosure_new(G_CALLBACK(accel_ctrl_o), NULL, NULL);
	gtk_accelerator_parse("<CTRL>O", &key, &mods);
	gtk_accel_group_connect(accel_group, key, mods, GTK_ACCEL_VISIBLE, closure);
	g_closure_unref(closure);

	closure = g_cclosure_new(G_CALLBACK(accel_ctrl_s), NULL, NULL);
	gtk_accelerator_parse("<CTRL>S", &key, &mods);
	gtk_accel_group_connect(accel_group, key, mods, GTK_ACCEL_VISIBLE, closure);
	g_closure_unref(closure);
	
	gtk_window_add_accel_group(GTK_WINDOW(edit), accel_group);

	// connect signals
	g_signal_connect((gpointer) edit, "delete_event", G_CALLBACK(edit_unload), NULL);
	g_signal_connect((gpointer) edit, "destroy", G_CALLBACK(edit_destroy), NULL);
	g_signal_connect((gpointer) new_button, "clicked", G_CALLBACK(new_button_click), NULL);
	g_signal_connect((gpointer) open_button, "clicked", G_CALLBACK(open_button_click), NULL);
	g_signal_connect((gpointer) save_button, "clicked", G_CALLBACK(save_button_click), NULL);
	g_signal_connect((gpointer) save_as_button, "clicked", G_CALLBACK(save_as_button_click), NULL);
	g_signal_connect((gpointer) about_button, "clicked", G_CALLBACK(about_button_click), NULL);
	g_signal_connect((gpointer) buffer, "changed", G_CALLBACK(buffer_change), NULL);

	return edit;
}
