//
// zedit - simple text editor
// Copyright (C) 2004 Bryan Beicker <tokiko@tokiko.net>
//
// This file is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License (Version 2)
// as published by the Free Software Foundation.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include <gtk/gtk.h>		// gtk
#include <sys/types.h>		// open()
#include <sys/stat.h>		// open()
#include <fcntl.h>		// open()
#include <stdio.h>		// BUFSIZE
#include <unistd.h>		// write()
#include "main.h"
#include "zedit.xpm"


//
// write a string to a file
//
int slap(char * path, char * data, int len)
{
	int fd;
	int bytes;
	int size;
	int count = 0;
	
	fd = open(path, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
	if (fd == -1) {
		return FALSE;
	}
	
	while (count < len) {
		size = len - count;
		if (size > BUFSIZ) {
			size = BUFSIZ;
		}
		bytes = write(fd, data + count, size);
		if (bytes == -1) {
			g_print("error writing file\n");
			close(fd);
			return FALSE;
		}
		count += bytes;
	}
	
	close(fd);
	return TRUE;
}


void about_dialog(char ** xpm, char * name, char * desc, char * copy)
{
	GtkWidget * dialog;
	dialog = gtk_dialog_new_with_buttons("about", GTK_WINDOW(edit), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CLOSE, GTK_RESPONSE_NONE, NULL);
	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
	
	GdkPixbuf * pixbuf;
	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) xpm);
	
	GtkWidget * table;
	table = gtk_table_new(3, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
	gtk_container_set_border_width(GTK_CONTAINER(table), 12);
	gtk_widget_show(table);

	GtkWidget * icon;
	icon = gtk_image_new_from_pixbuf(pixbuf);
	gdk_pixbuf_unref(pixbuf);
	gtk_misc_set_alignment(GTK_MISC(icon), 1, 0.5);
	gtk_widget_show(icon);
	gtk_table_attach(GTK_TABLE(table), icon, 0, 1, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 4, 2);
	
	gchar * tmp;
	GtkWidget * version;
	version = gtk_label_new(NULL);
	tmp = g_strdup_printf("<span weight=\"bold\" size=\"larger\">%s</span>", name);
	gtk_label_set_text(GTK_LABEL(version), tmp);
	g_free(tmp);
	gtk_misc_set_alignment(GTK_MISC(version), 0, 0.5);
	gtk_label_set_use_markup(GTK_LABEL(version), TRUE);
	gtk_widget_show(version);
	gtk_table_attach(GTK_TABLE(table), version, 1, 2, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	
	GtkWidget * description;
	description = gtk_label_new(desc);
	gtk_widget_show(description);
	gtk_table_attach(GTK_TABLE(table), description, 0, 2, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	
	GtkWidget * copyright;
	tmp = g_strdup_printf("<span size=\"smaller\">%s</span>", copy);
	copyright = gtk_label_new(tmp);
	g_free(tmp);
	gtk_label_set_use_markup(GTK_LABEL(copyright), TRUE);
	gtk_widget_show(copyright);
	gtk_table_attach(GTK_TABLE(table), copyright, 0, 2, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), table);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}


void main_load()
{
	file_name = NULL;
	dirty = FALSE;
}


//
// ask to save the file if dirty
//
//   return TRUE if the file is still dirty (they canceled the save as dialog)
//   return FALSE if the file was saved
//
gboolean dirty_check()
{
	GtkWidget * dialog;

	if (dirty) {
		dialog = gtk_message_dialog_new(GTK_WINDOW(edit), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "save changes?");
		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_YES) {
			save_button_click(NULL, NULL, NULL);
			if (dirty) {
				gtk_widget_destroy(dialog);
				return TRUE;
			}
		}
		gtk_widget_destroy(dialog);
	}
	
	return FALSE;
}


//
// window manager is trying to close the window
//
gboolean edit_unload(GtkWidget * window, GdkEvent * event, gpointer data)
{
	if (dirty_check()) {
		// prevent the window from closing
		return TRUE;
	}
	
	// let the window close
	return FALSE;
}


//
// exit the program when the window dies
//
void edit_destroy(GtkWidget * window, gpointer data)
{
	gtk_main_quit();
}


gboolean new_button_click(GtkWidget * widget, GdkEventButton * event, gpointer data)
{
	if (dirty_check()) {
		return TRUE;
	}
	
	gtk_text_buffer_set_text(buffer, "", -1);
	g_free(file_name);
	file_name = NULL;
	dirty = FALSE;
	gtk_window_set_title(GTK_WINDOW(edit), "zedit");

	return TRUE;
}


void open_file(char * path)
{
	GError * e;
	gsize len;
	gchar * body;

	g_free(file_name);
	file_name = path;
	
	g_file_get_contents(path, &body, &len, &e);
	gtk_text_buffer_set_text(buffer, body, len);
	g_free(body);
	
	gtk_window_set_title(GTK_WINDOW(edit), file_name);
	dirty = FALSE;
}


gboolean open_button_click(GtkWidget * widget, GdkEventButton * event, gpointer data)
{
	GtkWidget * dialog;
	gchar * path;
	
	if (dirty_check()) {
		return TRUE;
	}

	dialog = gtk_file_chooser_dialog_new("open", GTK_WINDOW(edit), GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		open_file(path);
	}
	gtk_widget_destroy(dialog);
	
	return TRUE;
}


gboolean save_button_click(GtkWidget * widget, GdkEventButton * event, gpointer data)
{
	GtkTextIter start, end;
	gchar * body;
	int len;

	if (file_name == NULL) {
		return save_as_button_click(NULL, NULL, NULL);
	}

	gtk_text_buffer_get_iter_at_offset(buffer, &start, 0);
	len = gtk_text_buffer_get_char_count(buffer);
	gtk_text_buffer_get_iter_at_offset(buffer, &end, len);
	body = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
	
	slap(file_name, body, len);
	gtk_window_set_title(GTK_WINDOW(edit), file_name);
	g_free(body);
	dirty = FALSE;

	return TRUE;
}


gboolean save_as_button_click(GtkWidget * widget, GdkEventButton * event, gpointer data)
{
	GtkWidget * dialog;
	
	dialog = gtk_file_chooser_dialog_new("save", GTK_WINDOW(edit), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		g_free(file_name);
		file_name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		save_button_click(NULL, NULL, NULL);
	}
	gtk_widget_destroy(dialog);
	
	return TRUE;
}


gboolean about_button_click(GtkWidget * widget, GdkEventButton * event, gpointer data)
{
	about_dialog(zedit_xpm, "zedit " VERSION, "simple text editor", "Copyright (C) 2004 Bryan Beicker");
	
	return TRUE;
}


void buffer_change(GtkEntry * widget, gpointer data)
{
	gchar * title;

	if (dirty == FALSE) {
		if (file_name != NULL) {
			title = g_strdup_printf("%s*", file_name);
			gtk_window_set_title(GTK_WINDOW(edit), title);
			g_free(title);
		}
		dirty = TRUE;
	}
}


void accel_ctrl_n(GtkAccelGroup * accelgroup, GObject * arg1, guint arg2, GdkModifierType arg3, gpointer data)
{
	new_button_click(NULL, NULL, NULL);
}


void accel_ctrl_o(GtkAccelGroup * accelgroup, GObject * arg1, guint arg2, GdkModifierType arg3, gpointer data)
{
	open_button_click(NULL, NULL, NULL);
}


void accel_ctrl_s(GtkAccelGroup * accelgroup, GObject * arg1, guint arg2, GdkModifierType arg3, gpointer data)
{
	save_button_click(NULL, NULL, NULL);
}
